/*
 *  Copyright  1995-2001, The AROS Development Team. All rights reserved.
 *  $Id: makecountry.c,v 1.18 2011/10/11 19:45:15 piru Exp $
 *
 *  Desc: Program that makes country files
 *  Lang: english
 */

#include <prefs/locale.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#define EC(__x) do { \
  uint32_t v = __x; \
  __x = ((v & 0xff000000) >> 24) | \
        ((v & 0xff0000) >> 8) | \
        ((v & 0xff00) << 8) | \
        ((v & 0xff) << 24); \
} while (0)

int _big_endian = 0; /* 0 == little endian, 1 == big endian */


struct CountryEntry
{
	char *ca_Name;
	struct CountryPrefs *ca_Data;
	unsigned char *ca_UTF8Name;
};

extern struct CountryPrefs
	andorraPrefs,
	australiaPrefs,
	belgiePrefs,
	belgiquePrefs,
	bosna_i_hercegovinaPrefs,
	brasilPrefs,
	bulgarijaPrefs,
	canadaPrefs,
	canada_francaisPrefs,
	ceskoPrefs,
	danmarkPrefs,
	deutschPrefs,
	eirePrefs,
	espanaPrefs,
	francePrefs,
	greatBritainPrefs,
	greecePrefs,
	icelandPrefs,
	iranPrefs,
	italiaPrefs,
	letzebuergPrefs,
	liechtensteinPrefs,
	lietuvaPrefs,
	magyarorszagPrefs,
	maltaPrefs,
	monacoPrefs,
	nederlandPrefs,
	norgePrefs,
	osterreichPrefs,
	polskaPrefs,
	portugalPrefs,
	romaniaPrefs,
	rossijaPrefs,
	san_marinoPrefs,
	schweizPrefs,
	slovakiaPrefs,
	slovenijaPrefs,
	suissePrefs,
	suomiPrefs,
	sverigePrefs,
	svizzeraPrefs,
	turkiyePrefs,
	ukrajinaPrefs,
	united_kingdomPrefs,
	united_statesPrefs;


/*  Please keep this in alphabetical order, ie the order of Latin 1 
 */
struct CountryEntry CountryArray[] =
{
	/* file name,            ptr,                       (optional) UTF8 country name */
	{ "andorra",             &andorraPrefs,             "Andorra"                      },
	{ "australia",           &australiaPrefs,           "Australia"                    },
	{ "belgie",              &belgiePrefs,              "Belgi""\xc3\xab"              },
//	{ "belgien",             &belgienPrefs,             "Belgien"                      },
	{ "belgique",            &belgiquePrefs,            "Belgique"                     },
	{ "bosna_i_hercegovina", &bosna_i_hercegovinaPrefs, "Bosna i Hercegovina"          }, /* cyrillic: "\xD0\x91\xD0\xBE\xD1\x81\xD0\xBD\xD0\xB0\x20\xD0\xB8\x20\xD0\xA5\xD0\xB5\xD1\x80\xD1\x86\xD0\xB5\xD0\xB3\xD0\xBE\xD0\xB2\xD0\xB8\xD0\xBD\xD0\xB0" */
	{ "brasil",              &brasilPrefs,              "Brasil"                       },                   
	{ "bulgarija",           &bulgarijaPrefs,           "B""\xc4\x83""lgariya"         }, /* cyrillic: "\xD1\x8A\xD0\xBB\xD0\xB3\xD0\xB0\xD1\x80\xD0\xB8\xD1\x8F" */
	{ "canada",              &canadaPrefs,              "Canada"                       },
	{ "canada_franais",     &canada_francaisPrefs,     "Canada (fran""\xc3\xa7""ais)" },
	{ "cesko",               &ceskoPrefs,               "\xc4\x8c""esko"               },
	{ "danmark",             &danmarkPrefs,             "Danmark"                      },
	{ "deutschland",         &deutschPrefs,             "Deutschland"                  },
	{ "ire",                &eirePrefs,                "\xc3\x89""ire"                },
	{ "espaa",              &espanaPrefs,              "Espa""\xc3\xb1""a"            },
	{ "france",              &francePrefs,              "France"                       },
	{ "great_britain",       &greatBritainPrefs,        "Great Britain"                },
	{ "greece",              &greecePrefs,              "Greece"                       },
	{ "iran",                &iranPrefs,                "\xc4\xaa""r""\xc4\x81n"       }, /* persian: "\xD8\xA7\xD9\x8A\xD8\xB1\xD8\xA7\xD9\x86" */
	{ "sland",              &icelandPrefs,             "\xc3\x8d""sland"              },
	{ "italia",              &italiaPrefs,              "Italia"                       },
	{ "ltzebuerg",          &letzebuergPrefs,          "L""\xc3\xab""tzebuerg"        },
//	{ "luxembourg",          &luxembourgPrefs,          "Luxembourg"                   },
//	{ "luxemburg",           &luxenburgPrefs,           "Luxenburg"                    },
	{ "liechtenstein",       &liechtensteinPrefs,       "Liechtenstein"                },
	{ "lietuva",             &lietuvaPrefs,             "Lietuva"                      },
	{ "magyarorszg",        &magyarorszagPrefs,        "Magyarorsz""\xc3\xa1""g"      },
	{ "malta",               &maltaPrefs,               "Malta"                        },
	{ "monaco",              &monacoPrefs,              "Monaco"                       },
	{ "nederland",           &nederlandPrefs,           "Nederland"                    },
	{ "norge",               &norgePrefs,               "Norge"                        },
	{ "sterreich",          &osterreichPrefs,          "\xc3\x96""sterreich"          },
	{ "polska",              &polskaPrefs,              "Polska"                       },
	{ "portugal",            &portugalPrefs,            "Portugal"                     },
	{ "romnia",             &romaniaPrefs,             "Rom""\xc3\xa2""nia"           },
/* XXX: Rossiya would be correct translitaration */ 
	{ "rossija",             &rossijaPrefs,             "Rossiya"                      }, /* cyrillic: "\xD0\xA0\xD0\xBE\xD1\x81\xD1\x81\xD0\xB8\xCC\x81\xD1\x8F" */
	{ "san_marino",          &san_marinoPrefs,          "San Marino"                   },
	{ "schweiz",             &schweizPrefs,             "Schweiz"                      },
/* XXX: not using correct way either */
	{ "slovakia",            &slovakiaPrefs,            "Slovensko"                    },
	{ "slovenija",           &slovenijaPrefs,           "Slovenija"                    },
	{ "suisse",              &suissePrefs,              "Suisse"                       },
	{ "suomi",               &suomiPrefs,               "Suomi"                        },
	{ "sverige",             &sverigePrefs,             "Sverige"                      },
	{ "svizzera",            &svizzeraPrefs,            "Svizzera"                     },
	{ "trkiye",             &turkiyePrefs,             "T""\xc3\xbc""rkiye"           },
	{ "ukrajina",            &ukrajinaPrefs,            "Ukrayina"                     }, /* cyrillic: "\xD0\xA3\xD0\xBA\xD1\x80\xD0\xB0\xD1\x97\xD0\xBD\xD0\xB0" */
	{ "united_kingdom",      &united_kingdomPrefs,      "United Kingdom"               },
	{ "united_states",       &united_statesPrefs,       "United States of America"     },
	{ NULL,                  NULL,                      NULL }
};


int doCountry(struct CountryPrefs *cp, char *progname, char *filename, char *utf8name)
{
	FILE *fp;
	uint32_t filelen = 0x212; /* filesize with CTRY chunk only (530 bytes) */
	uint32_t utf8len = 0;


	/*  This is equivalent to the start of the catalog file.
	 *  It is a series of strings, so that the endianness is
	 *  correct either way
	 */
	char preamble[] =
	{
		"FORM"
			"\x00\x00\x02\x12" /* default file size of 530 bytes when only CTRY chunk is written */
		"PREF"
		"PRHD" 
			"\x00\x00\x00\x06"
			"\x00\x00\x00\x00\x00\x00"
		"CTRY" 
			"\x00\x00\x01\xF8"        
	};

	char utf8name_chunk[] =
	{
		"CTRU"
			"\x00\x00\x00\x00"
	};


	fp = fopen(filename, "w");

	if(fp == NULL)
	{
		printf("%s: Could not open file %s\n", progname, filename);
		return (20);
	}

	if (utf8name)
	{
		utf8len = strlen(utf8name) + 1; /* including '\0' */

		filelen += 8 + utf8len; /* 8 for the chunk itself */

		if (utf8len % 2) 
		{
			filelen++;
		}

		if (!_big_endian)
		{
			EC(filelen);
		}

		*(uint32_t *)(preamble+4) = filelen;
	}

	/*  Write the remaining preamble...
	 *    FORM 0xXXXXXXXX
	 *    PREF
	 *    PRHD 0x00000006
	 *    CTRY 0x000001F8
	 */
	if(fwrite(preamble, 34, 1, fp) < 1)
	{
		printf("%s: Write error during preable of %s.\n", progname, filename);
		fclose(fp);
		return(20);
	}

	if (!_big_endian)
	{
		/*  We have to convert the endianness of this data,
		 *  thankfully there are only two fields which this applies
		 *  to.
		 */
		EC(cp->cp_CountryCode);
		EC(cp->cp_TelephoneCode);
	}

	if(fwrite(cp, sizeof(struct CountryPrefs), 1, fp) < 1)
	{
		printf("%s: Write error during data for %s.\n", progname, filename);
		fclose(fp);

		return(20);
	}


	/*  Write optional UTF8 name chunk
	 */
	if (utf8name)
	{
		filelen = utf8len; /* size of chunk */

		if (utf8len % 2) 
		{
			filelen++; /*  maybe not required, but it doesn't hurt either */
		}

		if (!_big_endian)
		{
			EC(filelen);
		}

		*(uint32_t *)(utf8name_chunk+4) = filelen;

		/*  Write utf8name chunk
		 *    CTRU 0xXXXXXXXX
		 */
		if(fwrite(utf8name_chunk, 8, 1, fp) < 1)
		{
			printf("%s: Write error during preable of %s.\n", progname, filename);
			fclose(fp);
			return(20);
		}

		if(fwrite(utf8name, utf8len, 1, fp) < 1)
		{
			printf("%s: Write error during utf8 name data of %s.\n", progname, filename);
			fclose(fp);
			return(20);
		}

		if (utf8len % 2)
		{
			if(fwrite("\0", 1, 1, fp) < 1)
			{
				printf("%s: Write error during aligment byte of %s.\n", progname, filename);
				fclose(fp);
				return(20);
			}
		}
	}

	fclose(fp);

	return 0;
}


int main(int argc, char **argv)
{
	int i,j,res;

	uint32_t eint = 0x000000FF;

	if (*(uint8_t *)&eint != 0xFF)
	{	
		_big_endian = 1;
	}

	if(argc < 3)
	{
		printf("%s: Wrong number of arguments\n", argv[0]);

		return(20);
	}

	for(i=2; i < argc; i++)
	{
		for(j=0; CountryArray[j].ca_Name != NULL; j++)
		{
			res = strcmp(CountryArray[j].ca_Name, argv[i]);
			
			if(res == 0)
			{
				unsigned char buffer[1024];

				strcpy(buffer, argv[1]);
				strcat(buffer, argv[i]);
				strcat(buffer, ".country");

				doCountry(CountryArray[j].ca_Data, argv[0], buffer, CountryArray[j].ca_UTF8Name);

				break;
			}
			#if 0
			/* stegerg: does not work because of '' in sterreich */
			/* If countryArray < argv[] don't bother searching */
			else 
			{
				if(res > 0)
				{
					break;
				}
			}
			#endif

		} /* for j */

	} /* for i */

	return 0;
}
